/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */

package org.ow2.petals.component.framework.clientserver.api.monitoring;

import java.util.Map;

import org.ow2.petals.component.framework.clientserver.api.monitoring.exception.MonitoringProbeNotInitializedException;
import org.ow2.petals.component.framework.clientserver.api.monitoring.exception.MonitoringProbeNotStartedException;
import org.ow2.petals.component.framework.clientserver.api.monitoring.exception.MonitoringServiceException;

/**
 * Client/Server interface of the service 'Monitoring' of a component based on the CDK
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public interface MonitoringService {

    //
    // --- Message exchange acceptor thread pool metrics
    //

    /**
     * @return The max size of the message exchange acceptor thread pool.
     * @throws MonitoringProbeNotInitializedException
     *             The probe is not initialized or shutdown.
     * @throws MonitoringServiceException
     */
    public int getMessageExchangeAcceptorThreadPoolMaxSize() throws MonitoringProbeNotInitializedException,
            MonitoringServiceException;

    /**
     * @return The current size of the message exchange acceptor thread pool.
     * @throws MonitoringProbeNotStartedException
     *             The probe is not started or stopped.
     * @throws MonitoringServiceException
     */
    public int getMessageExchangeAcceptorThreadPoolCurrentSize() throws MonitoringProbeNotStartedException,
            MonitoringServiceException;

    /**
     * @return The number of message exchange acceptor currently running.
     * @throws MonitoringProbeNotStartedException
     *             The probe is not initialized or shutdown.
     * @throws MonitoringServiceException
     */
    public long getMessageExchangeAcceptorCurrentWorking() throws MonitoringProbeNotStartedException,
            MonitoringServiceException;

    /**
     * @return The max number of message exchange acceptor concurrently running since the last start of the component.
     * @throws MonitoringProbeNotStartedException
     *             The probe is not initialized or shutdown.
     * @throws MonitoringServiceException
     */
    public long getMessageExchangeAcceptorMaxWorking() throws MonitoringProbeNotInitializedException,
            MonitoringServiceException;

    /**
     * @return The absolute durations, in nanoseconds, of the message exchange acceptors:
     *         <ol>
     *         <li>the first long value is the max response time,</li>
     *         <li>the second long value is the average of response times,</li>
     *         <li>the third long value is the min response time.</li>
     *         </ol>
     * @throws MonitoringProbeNotInitializedException
     *             The probe is not initialized or shutdown.
     * @throws MonitoringServiceException
     */
    public Long[] getMessageExchangeAcceptorAbsoluteDurations() throws MonitoringProbeNotInitializedException,
            MonitoringServiceException;

    /**
     * @return The relative durations, in nanoseconds, of the message exchange acceptors:
     *         <ol>
     *         <li>the first long value is the max response time,</li>
     *         <li>the second long value is the average of response times,</li>
     *         <li>the third long value is the min response time,</li>
     *         <li>the fourth long value is the 10-percentile response time,</li>
     *         <li>the fifth long value is the 50-percentile response time,</li>
     *         <li>the sixth long value is the 90-percentile response time.</li>
     *         </ol>
     * @throws MonitoringProbeNotStartedException
     *             The probe is not started or stopped.
     * @throws MonitoringServiceException
     */
    public Long[] getMessageExchangeAcceptorRelativeDurations() throws MonitoringProbeNotStartedException,
            MonitoringServiceException;

    //
    // --- Message exchange processor thread pool metrics
    //

    /**
     * @return The current max size of the message exchange processor thread pool.
     * @throws MonitoringServiceException
     */
    public long getMessageExchangeProcessorThreadPoolMaxSize() throws MonitoringServiceException;

    /**
     * @return The current min size of the message exchange processor thread pool.
     * @throws MonitoringServiceException
     */
    public long getMessageExchangeProcessorThreadPoolMinSize() throws MonitoringServiceException;

    /**
     * @return The max number of active threads of the message exchange processor thread pool.
     * @throws MonitoringProbeNotInitializedException
     *             The probe is not initialized or shutdown.
     * @throws MonitoringServiceException
     */
    public long getMessageExchangeProcessorThreadPoolActiveThreadsMax()
            throws MonitoringProbeNotInitializedException, MonitoringServiceException;

    /**
     * @return The current number of active threads of the message exchange processor thread pool.
     * @throws MonitoringProbeNotStartedException
     *             The probe is not started or stopped.
     * @throws MonitoringServiceException
     */
    public long getMessageExchangeProcessorThreadPoolActiveThreadsCurrent()
            throws MonitoringProbeNotStartedException, MonitoringServiceException;

    /**
     * @return The max number of idle threads of the message exchange processor thread pool.
     * @throws MonitoringProbeNotInitializedException
     *             The probe is not initialized or shutdown.
     * @throws MonitoringServiceException
     */
    public long getMessageExchangeProcessorThreadPoolIdleThreadsMax() throws MonitoringProbeNotInitializedException,
            MonitoringServiceException;

    /**
     * @return The current number of idle threads of the message exchange processor thread pool.
     * @throws MonitoringProbeNotStartedException
     *             The probe is not started or stopped.
     * @throws MonitoringServiceException
     */
    public long getMessageExchangeProcessorThreadPoolIdleThreadsCurrent() throws MonitoringProbeNotStartedException,
            MonitoringServiceException;

    /**
     * @return The max number of enqueued tasks into the message exchange processor thread pool.
     * @throws MonitoringProbeNotInitializedException
     *             The probe is not initialized or shutdown.
     * @throws MonitoringServiceException
     */
    public long getMessageExchangeProcessorThreadPoolQueuedRequestsMax() throws MonitoringProbeNotInitializedException,
            MonitoringServiceException;

    /**
     * @return The current number of enqueued tasks into the message exchange processor thread pool.
     * @throws MonitoringProbeNotStartedException
     *             The probe is not started or stopped.
     * @throws MonitoringServiceException
     */
    public long getMessageExchangeProcessorThreadPoolQueuedRequestsCurrent() throws MonitoringProbeNotStartedException,
            MonitoringServiceException;




    //
    // --- Message exchange processors (processing duration)
    //

    /**
     * @return The absolute durations of the message exchange processors:
     *         <ol>
     *         <li>the first long value is the max response time,</li>
     *         <li>the second long value is the average of response times,</li>
     *         <li>the third long value is the min response time.</li>
     *         </ol>
     * @throws MonitoringProbeNotInitializedException
     *             The probe is not initialized or shutdown.
     * @throws MonitoringServiceException
     */
    public Long[] getMessageExchangeProcessorAbsoluteDurations() throws MonitoringProbeNotInitializedException,
            MonitoringServiceException;

    /**
     * @return The relative durations of the message exchange processors:
     *         <ol>
     *         <li>the first long value is the max response time,</li>
     *         <li>the second long value is the average of response times,</li>
     *         <li>the third long value is the min response time,</li>
     *         <li>the fourth long value is the 10-percentile response time,</li>
     *         <li>the fifth long value is the 50-percentile response time,</li>
     *         <li>the sixth long value is the 90-percentile response time.</li>
     *         </ol>
     * @throws MonitoringProbeNotStartedException
     *             The probe is not started or stopped.
     * @throws MonitoringServiceException
     */
    public Long[] getMessageExchangeProcessorRelativeDurations() throws MonitoringProbeNotStartedException,
            MonitoringServiceException;

    //
    // --- Response times of service providers
    //

    /**
     * <p>
     * Returns the response time values of service providers since the last start.
     * </p>
     * <p>
     * The map key is structured as a {@link String} array where:
     * </p>
     * <ul>
     * <li>1st: the interface name (as qualified name) of the invoked service,</li>
     * <li>2nd: the service name (as qualified name) of the invoked service,</li>
     * <li>3rd: the operation invoked (as qualified name),</li>
     * <li>4th: the message exchange pattern</li>
     * <li>5th: the execution status (see <code>org.ow2.petals.probes.api.enums.ExecutionStatus</code>).</li>
     * </ul>
     * <p>
     * Each value of the map entry is an array of response times where:
     * </p>
     * <ul>
     * <li>the first value is the max response time,</li>
     * <li>the second value is the average,</li>
     * <li>the third value is the min value.</li>
     * </ul>
     * 
     * @return The absolute response time values.
     * @throws MonitoringProbeNotStartedException
     *             The probe is not started or stopped.
     * @throws MonitoringServiceException
     */
    public Map<String[], Long[]> getServiceProviderInvocationsResponseTimeAbs()
            throws MonitoringProbeNotInitializedException, MonitoringServiceException;

    /**
     * <p>
     * Returns the response time values of service providers on a time sample.
     * </p>
     * <p>
     * The map key is an array containing:
     * </p>
     * <ul>
     * <li>1st: the interface name (as qualified name) of the invoked service,</li>
     * <li>2nd: the service name (as qualified name) of the invoked service,</li>
     * <li>3rd: the operation invoked (as qualified name),</li>
     * <li>4th: the message exchange pattern</li>
     * <li>5th: the execution status (see <code>org.ow2.petals.probes.api.enums.ExecutionStatus</code>).</li>
     * </ul>
     * <p>
     * Each value of the map entry is an array of response times where:
     * </p>
     * <ul>
     * <li>the first value is the max response time on the time sample,</li>
     * <li>the second value is the average,</li>
     * <li>the third value is the min value,</li>
     * <li>the fourth value is the 10-percentile,</li>
     * <li>the fifth value is the 50-percentile,</li>
     * <li>the sixth value is the 90-percentile.</li>
     * </ul>
     * 
     * @return The response time values on a time sample.
     * @throws MonitoringProbeNotStartedException
     *             The probe is not started or stopped.
     * @throws MonitoringServiceException
     */
    public Map<String[], Long[]> getServiceProviderInvocationsResponseTimeRel()
            throws MonitoringProbeNotStartedException, MonitoringServiceException;

    /**
     * <p>
     * Returns the response time values of service providers on a time sample group by interface, service,operation and
     * execution status
     * </p>
     * <p>
     * The map key is an array containing:
     * </p>
     * <ul>
     * <li>1st: the interface name (as qualified name) of the invoked service,</li>
     * <li>2nd: the service name (as qualified name) of the invoked service,</li>
     * <li>3rd: the operation invoked (as qualified name),</li>
     * <li>4th: the execution status (see <code>org.ow2.petals.probes.api.enums.ExecutionStatus</code>).</li>
     * </ul>
     * <p>
     * Each value of the map entry is an array of response times where:
     * </p>
     * <ul>
     * <li>the first value is the max response time on the time sample,</li>
     * <li>the second value is the average,</li>
     * <li>the third value is the min value,</li>
     * <li>the fourth value is the 10-percentile,</li>
     * <li>the fifth value is the 50-percentile,</li>
     * <li>the sixth value is the 90-percentile.</li>
     * </ul>
     * 
     * @return The response time values. All execution status are contained in the map, even if set to 0.
     * 
     * @throws MonitoringProbeNotStartedException
     * @throws MonitoringServiceException
     */
    public Map<String[], Long[]> getServiceProviderInvocationsResponseTimeRelGroupedByItfSvcOpAndExecStatus()
            throws MonitoringProbeNotStartedException, MonitoringServiceException;

    //
    // --- Counters of service providers
    //

    /**
     * <p>
     * Returns the invocation number of service providers.
     * </p>
     * <p>
     * The map key is an array containing:
     * </p>
     * <ul>
     * <li>1st: the interface name (as qualified name) of the invoked service,</li>
     * <li>2nd: the service name (as qualified name) of the invoked service,</li>
     * <li>3rd: the operation invoked (as qualified name),</li>
     * <li>4th: the message exchange pattern</li>
     * <li>5th: the execution status (see <code>org.ow2.petals.probes.api.enums.ExecutionStatus</code>).</li>
     * </ul>
     * <p>
     * Each value of the map entry is the number of invocations
     * </p>
     * 
     * @return The invocation number of service providers.
     * @throws MonitoringProbeNotInitializedException
     *             The probe is not initialized or shutdown.
     * @throws MonitoringServiceException
     */
    public Map<String[], Long> getServiceProviderInvocations() throws MonitoringProbeNotInitializedException,
            MonitoringServiceException;
}
